package UDP::Model::Fertility;

use Moose;

has 'alpha' => ( is => 'rw', isa => 'Num', default => 0.1 );
has 'frequency' => ( is => 'rw', isa => 'HashRef', default => sub {my %h = (); return \%h;} );
has 'freq_contribution' => ( is => 'rw', isa => 'Str', default => 'linear'); 

sub load_freq_from_file {
    my ($self, $filename) = @_;
    open (SCORES, "<:utf8", $filename) or die "The file $filename doesn't exist.";
    while (<SCORES>) {
        chomp;
        my ($form, $freq) = split /\t/;
        $self->frequency->{$form} = $freq;
    }
}


sub initialize {
    my ($self, $corpus) = @_;
    my %word_count;
    my $total_count;
    foreach my $sentence ($corpus->get_sentences) {
        foreach my $node ($sentence->get_nodes) {
            $word_count{lc($node->form)}++;
            $total_count++;
        }
    }
    foreach my $word (keys %word_count) {
        $self->frequency->{$word} = $word_count{$word} / $total_count;
    }
}



sub compute_counts_and_logprob {
    my ($self, $type, $node, $diff_count, $diff_logprob, $count) = @_;

    my $number_of_children = scalar $node->get_children;
    my $number_of_left_children = scalar grep{$_->ord < $node->ord} $node->get_children();
    my $number_of_right_children = $number_of_children - $number_of_left_children;
    my $tag = $node->tag;

    my $prior_prob = 1 / 2 **($number_of_children + 1);

    my $alpha = $self->alpha;
    if ($self->freq_contribution eq 'log') {
        $alpha *= log(1/$self->frequency->{lc($node->form)});
    }
    elsif ($self->freq_contribution eq 'linear') {
        $alpha /= $self->frequency->{lc($node->form)};
    }

    my $ch_p_label = "f_p $number_of_left_children | $tag | $number_of_right_children";
    my $p_label = "p $tag";

    if ($type eq 'del') {
       $diff_count->{$ch_p_label}--;
       $diff_count->{$p_label}--;
       $$diff_logprob -= log(($count->{$ch_p_label} || 0) + ($diff_count->{$ch_p_label} || 0) + $prior_prob * $alpha);
       $$diff_logprob += log(($count->{$p_label} || 0) + ($diff_count->{$p_label} || 0) + $alpha);
       
    }
    elsif ($type eq 'ins') {
        $$diff_logprob += log(($count->{$ch_p_label} || 0) + ($diff_count->{$ch_p_label} || 0) + $prior_prob * $alpha);
        $$diff_logprob -= log(($count->{$p_label} || 0) + ($diff_count->{$p_label} || 0) + $alpha);
        $diff_count->{$ch_p_label}++;
        $diff_count->{$p_label}++;
    }
}

1;

