package UDP::Model::LexicalEdge;

use Moose;

has 'alpha' => ( is => 'rw', isa => 'Num', default => 1 );
has 'classes' => ( is => 'rw', isa => 'Int', default => 100 );

sub initialize {
    my ($self, $corpus) = @_;
    my %form_exists;
    foreach my $sentence ($corpus->get_sentences) {
        foreach my $node ($sentence->get_nodes) {
            $form_exists{lc($node->form)} = 1;
        }
    }
    $self->classes(scalar keys %form_exists);
}

sub compute_counts_and_logprob {
    my ($self, $type, $node, $diff_count, $diff_logprob, $count) = @_;
    my $parent = $node->parent;
    my $direction = $node->ord < $parent->ord ? 'L' : 'R';
    my $context_label = "lex_context ".$node->tag." ".lc($parent->form)." ".$parent->tag." ".$direction;
    my $edge_label = "lex_edge ".lc($node->form)." ".$node->tag." ".lc($parent->form)." ".$parent->tag." ".$direction;
    if ($type eq 'del') {
        $diff_count->{$edge_label}--;
        $diff_count->{$context_label}--;
        $$diff_logprob -= log(($count->{$edge_label} || 0) + ($diff_count->{$edge_label} || 0) + $self->alpha);
        $$diff_logprob += log(($count->{$context_label} || 0) + ($diff_count->{$context_label} || 0) + $self->alpha * $self->classes);
    }
    elsif ($type eq 'ins') {
        $$diff_logprob += log(($count->{$edge_label} || 0) + ($diff_count->{$edge_label} || 0) + $self->alpha);
        $$diff_logprob -= log(($count->{$context_label} || 0) + ($diff_count->{$context_label} || 0) + $self->alpha * $self->classes);
        $diff_count->{$edge_label}++;
        $diff_count->{$context_label}++;
    }
}

1;

