package UDP::Node;

use Moose;

has 'parent'    => ( is => 'rw', isa => 'UDP::Node' );
has 'children'  => ( is => 'rw', isa => 'ArrayRef', default => sub { [] } );
has 'is_aux'    => ( is => 'rw', isa => 'Bool', default => 0 );
has 'tag'       => ( is => 'rw', isa => 'Str' );
has 'form'      => ( is => 'rw', isa => 'Str' );
has 'ord'       => ( is => 'rw', isa => 'Int' );
has 'leftmost'  => ( is => 'rw', isa => 'Int' );
has 'rightmost' => ( is => 'rw', isa => 'Int' );
has 'sentence'  => ( is => 'rw', isa => 'UDP::Sentence' );


sub remove_child {
    my ($self, $child) = @_;
    my $last = pop @{$self->children};
    return if $last eq $child;
    foreach my $i (0 .. $#{$self->children}) {
        if ($self->children->[$i] eq $child) {
            $self->children->[$i] = $last;
            return;
        }
    }
}

sub get_children {
    my ($self) = @_;
    return @{$self->children};
}

sub set_children {
    my ($self, $children) = @_;
    $self->children($children);
    # update leftmost and rightmost ord
    my $my_ord = $self->ord;
    my $left_child_ord = $children->[0] ? $children->[0]->leftmost : $self->ord;
    my $right_child_ord = $children->[0] ? $children->[$#$children]->rightmost : $self->ord;
    $self->leftmost($my_ord < $left_child_ord ? $my_ord : $left_child_ord);
    $self->rightmost($my_ord > $right_child_ord ? $my_ord : $right_child_ord);
}

sub get_descendants {
    my ($self) = @_;
    my @nodes = @{$self->sentence->nodes};
    return @nodes[($self->leftmost - 1) .. ($self->rightmost - 1)];
}

sub get_unordered_descendants {
    my ($self) = @_;
#    print STDERR join " ", map {$_->tag} $self->get_children;
#    print STDERR "     ";
#    print STDERR join " ", map {$_->ord} $self->get_children;
#    print STDERR "\n" if $self->get_children;
    return ($self, map {$_->get_unordered_descendants} $self->get_children);
}

1;

